<?php

defined('_JEXEC') or die;

class FinderIndexerStemmerPorter_Ru extends FinderIndexerStemmer
{
    var $VOWEL = '/аеиоуыэюя/u';
    var $PERFECTIVEGROUND = '/((ив|ивши|ившись|ыв|ывши|ывшись)|((?<=[ая])(в|вши|вшись)))$/u';
    var $REFLEXIVE = '/(с[яь])$/u';
    var $ADJECTIVE = '/(ее|ие|ые|ое|ими|ыми|ей|ий|ый|ой|ем|им|ым|ом|его|ого|еых|ую|юю|ая|яя|ою|ею)$/u';
    var $PARTICIPLE = '/((ивш|ывш|ующ)|((?<=[ая])(ем|нн|вш|ющ|щ)))$/u';
    var $VERB = '/((ила|ыла|ена|ейте|уйте|ите|или|ыли|ей|уй|ил|ыл|им|ым|ены|ить|ыть|ишь|ую|ю)|((?<=[ая])(ла|на|ете|йте|ли|й|л|ем|н|ло|но|ет|ют|ны|ть|ешь|нно)))$/u';
    var $NOUN = '/(а|ев|ов|ие|ье|е|иями|ями|ами|еи|ии|и|ией|ей|ой|ий|й|и|ы|ь|ию|ью|ю|ия|ья|я)$/u';
    var $RVRE = '/^(.*?[аеиоуыэюя])(.*)$/u';
    var $DERIVATIONAL = '/[^аеиоуыэюя][аеиоуыэюя]+[^аеиоуыэюя]+[аеиоуыэюя].*(?<=о)сть?$/u';

	
	/**
	 * Method to stem a token and return the root.
	 *
	 * @param   string  $token  The token to stem.
	 * @param   string  $lang   The language of the token.
	 *
	 * @return  string  The root token.
	 *
	 * @since   2.5
	 */
	public function stem($token, $lang)
	{
		// Check if the token is long enough to merit stemming.
		if (strlen($token) <= 2)
		{
			return $token;
		}

		// Check if the language is English or All.
		if ($lang !== 'ru' && $lang != '*')
		{
			return $token;
		}

		// Stem the token if it is not in the cache.
		if (!isset($this->cache[$lang][$token]))
		{
			// Stem the token.
			$result = $token;
			$result = preg_replace("/ё/u","е",$token);
			# Check against cache of stemmed words
			if (isset($this->cache[$lang][$result]))
			{
				return $this->cache[$lang][$result];
			}
			$stem = $result;
			do {
			  if (!preg_match($this->RVRE, $result, $p)) break;
			  $start = $p[1];
			  $RV = $p[2];
			  if (!$RV) break;

			  # Step 1
			  if (!$this->s($RV, $this->PERFECTIVEGROUND, '')) {
				  $this->s($RV, $this->REFLEXIVE, '');

				  if ($this->s($RV, $this->ADJECTIVE, '')) {
					  $this->s($RV, $this->PARTICIPLE, '');
				  } else {
					  if (!$this->s($RV, $this->VERB, ''))
						  $this->s($RV, $this->NOUN, '');
				  }
			  }

			  # Step 2
			  $this->s($RV, '/и$/u', '');

			  # Step 3
			  if ($this->m($RV, $this->DERIVATIONAL))
				  $this->s($RV, '/ость?$/u', '');

			  # Step 4
			  if (!$this->s($RV, '/ь$/u', '')) {
				  $this->s($RV, '/ейше?/u', '');
				  $this->s($RV, '/нн$/u', 'н');
			  }

			  $stem = $start.$RV;
			} while(false);

			// Add the token to the cache.
			$this->cache[$lang][$token] = $stem;
		}

		return $this->cache[$lang][$token];
	}

    function s(&$s, $re, $to)
    {
        $orig = $s;
        $s = preg_replace($re, $to, $s);
        return $orig !== $s;
    }

    function m($s, $re)
    {
        return preg_match($re, $s);
    }
}
